#pragma once
double* create_grid_even(int n, double a, double b);
double f(double x, double y);
double* f_eq_form(double x);
double** runge_adams(double eps, double y0, double a, double b, double(*f)(double, double), int* n_, int* len_h);
double* adams_moulten(double* start_points, double a, double b, double(*f)(double, double), int n);
double uglify_initial(double initial, int max_procent, int min_procent);